/*
	*
	* Copyright 2020 Britanicus <marcusbritanicus@gmail.com>
	*
	* This file is a part of DesQ project (https://gitlab.com/desq/)
	*

	*
	* This file was a part of IonShell (https://gitlab.com/cubocore/ion/ionde)
	* Suitable modifications have been done to meet the needs of DesQ.
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* ( at your option ) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#include <QDebug>

#include "DesQRegistry.hpp"
#include "DesQRegistryImpl.hpp"

#include "DesQLayerShell.hpp"
#include "DesQWindowManager.hpp"

#include "qwayland-wayfire-shell-unstable-v2.h"
#include "qwayland-wlr-layer-shell-unstable-v1.h"
#include "qwayland-wlr-foreign-toplevel-management-unstable-v1.h"

DesQWaylandRegistryPrivate::DesQWaylandRegistryPrivate( wl_display *wlDisplay, DesQWaylandRegistry *ptr ) : q_ptr( ptr ), m_wlDisplay( wlDisplay ) {
};

DesQWaylandRegistryPrivate::~DesQWaylandRegistryPrivate() {

	wl_registry_destroy( object() );
};

void DesQWaylandRegistryPrivate::initialize() {

	init( wl_display_get_registry( m_wlDisplay ) );
	wl_display_roundtrip( m_wlDisplay ); // Wait until the server responds
};

void DesQWaylandRegistryPrivate::registry_global( uint32_t name, const QString &interface, uint32_t version ) {

	Q_Q( DesQWaylandRegistry );

	if ( interface == wl_compositor_interface.name ) {
		m_wlCompositor = static_cast<wl_compositor*>( bind( name, &wl_compositor_interface, version ) );

		if (!m_wlCompositor)
			emit q->errorOccured( DesQWaylandRegistry::EmptyCompositor );
	}

	else if ( interface == wl_seat_interface.name ) {
		m_wlSeat = static_cast<wl_seat*>( bind( name, &wl_seat_interface, version ) );

		if (!m_wlSeat)
			emit q->errorOccured( DesQWaylandRegistry::EmptySeat );
	}

	else if (interface == zwlr_layer_shell_v1_interface.name) {
		m_wlrLayerShell = static_cast<zwlr_layer_shell_v1*>( bind( name, &zwlr_layer_shell_v1_interface, 4 ) );

		if (!m_wlrLayerShell)
			emit q->errorOccured( DesQWaylandRegistry::EmptyDesQLayerShell );

		else
			m_layerShell = new DesQLayerShell( m_wlrLayerShell );
	}

	else if ( interface == zwlr_foreign_toplevel_manager_v1_interface.name ) {
		m_wlrToplevelManager = static_cast<zwlr_foreign_toplevel_manager_v1 *>( bind( name, &zwlr_foreign_toplevel_manager_v1_interface, 3 ) );

		if ( !m_wlrToplevelManager )
			emit q->errorOccured( DesQWaylandRegistry::EmptyToplevelManager );

		else
			m_winMgr = new DesQWindowManager( m_wlrToplevelManager, m_wlSeat );
	}

	else if (interface == zwf_shell_manager_v2_interface.name) {

		m_wayfireShell = static_cast<zwf_shell_manager_v2*>( bind( name, &zwf_shell_manager_v2_interface, 1 ) );
	}
};

void DesQWaylandRegistryPrivate::registry_global_remove( uint32_t name ) {
};

/* ============================================================================ */

DesQWaylandRegistry::DesQWaylandRegistry( wl_display *display ) : QObject() {

	d_ptr = new DesQWaylandRegistryPrivate( display, this );
};

DesQWaylandRegistry::~DesQWaylandRegistry() {

};

void DesQWaylandRegistry::initialize() {

	Q_D( DesQWaylandRegistry );
	d->initialize();
};

wl_display *DesQWaylandRegistry::wlDisplay() const {

	Q_D( const DesQWaylandRegistry );
	return d->m_wlDisplay;
};

wl_seat *DesQWaylandRegistry::wlSeat() const {

	Q_D( const DesQWaylandRegistry );
	return d->m_wlSeat;
};

wl_compositor *DesQWaylandRegistry::wlCompositor() const {

	Q_D( const DesQWaylandRegistry );
	return d->m_wlCompositor;
};

zwlr_layer_shell_v1 *DesQWaylandRegistry::wlrLayerShell() const {

	Q_D( const DesQWaylandRegistry );
	return d->m_wlrLayerShell;
};

zwlr_foreign_toplevel_manager_v1 *DesQWaylandRegistry::wlrWindowManager() const {

	Q_D( const DesQWaylandRegistry );
	return d->m_wlrToplevelManager;
};

zwf_shell_manager_v2 *DesQWaylandRegistry::wayfireShell() const {

	Q_D( const DesQWaylandRegistry );
	return d->m_wayfireShell;
};

DesQLayerShell *DesQWaylandRegistry::layerShell() const {

	Q_D( const DesQWaylandRegistry );
	return d->m_layerShell;
};

DesQWindowManager *DesQWaylandRegistry::windowManager() const {

	Q_D( const DesQWaylandRegistry );
	return d->m_winMgr;
};
